/* -*- mode: c++; c-basic-offset: 4; -*- */
#include "SigFlag.hh"
#include "Interval.hh"
#include "Time.hh"
#include "lsmp_con.hh"
#include <iostream>
#include <string>
#include <cmath>
#include <cstdlib>

using namespace std;

//======================================  Command line syntax
static void
syntax(void) {
    cout << "smlatency measures the latency of frames arriving in a" << endl;
    cout << "specified partition. The time is measured from the gps" << endl;
    cout << "time stamp carried in the partition to the current gps" << endl;
    cout << "time inferred from the system clock. Note that a misset" << endl;
    cout << "system clock will add an unknown offset to the latency" << endl;
    cout << " " << endl;
    cout << "The command line syntax is:" << endl;
    cout << " " << endl;
    cout << "   smlatency [--help] [--average <n>] <partition>" << endl;
}

//======================================  Main program.
int
main(int argc, const char* argv[]) {
    SigFlag mTerm(SIGTERM);
    mTerm.add(SIGINT);
    mTerm.zero();

    //----------------------------------  Parse the command args
    int nAvg = 10;
    const char* part=getenv("LIGOSMPART");
    for (int i=1; i<argc; i++) {
	string argi = argv[i];
	if (argi == "--average") {
	    nAvg = strtol(argv[++i], 0, 0);
	}
	else if (argi == "--help") {
	    syntax();
	    return 0;	    
	}
	else if (i != argc-1) {
	    cerr << "Unrecognized command argument: " << argi << endl;
	    syntax();
	    return 1;
	}
    }

    //----------------------------------  Check partition name.
    if (argc > 1) part = argv[argc-1];
    if (!part) {
	cerr << "Partition name not specified." << endl;
	syntax();
	return 2;
    }

    //----------------------------------  Connect to the partition.
    LSMP_CON cons(part);
    if (!cons.attached()) {
	cerr << "Unable to open partition: " << part << endl;
	return 3;
    }

    double Avg(0.0), Sig(0.0), Max(0.0);
    int    iAvg(0);
    while (!mTerm) {
	cons.get_buffer();
	Time t=Time(cons.getEvtID());
	cons.free_buffer();
	double dt = double(Now() - t);
	Avg += dt;
	Sig += dt*dt;
	if (Max < dt) Max = dt;
	if (++iAvg >= nAvg) {
	    Avg /= double(iAvg);
	    Sig  = Sig / double(iAvg) - Avg * Avg;
	    if (Sig <= 0) Sig = 0;
	    else          Sig = sqrt(Sig);
	    cout << "Partition: " << part << " latency, average = " << Avg 
		 << " sigma = " << Sig << " max = " << Max << endl;
	    Avg = 0; iAvg = 0; Sig = 0; Max = 0.0;
	}
    }
    return 0;
}
